==> gitlab_17.0.8.orig.tar.bz2: gitlab-foss-v17.0.8/app/services/packages/npm/process_package_file_service.rb <==
# frozen_string_literal: true

module Packages
  module Npm
    class ProcessPackageFileService
      ExtractionError = Class.new(StandardError)
      PACKAGE_JSON_ENTRY_PATH = 'package/package.json'
      MAX_FILE_SIZE = 4.megabytes

      delegate :package, to: :package_file

      def initialize(package_file)
        @package_file = package_file
      end

      def execute
        raise ExtractionError, 'invalid package file' unless valid_package_file?

        with_package_json_entry do |entry|
          raise ExtractionError, 'package.json not found' unless entry
          raise ExtractionError, 'package.json file too large' if entry.size > MAX_FILE_SIZE

          ::Packages::Npm::CheckManifestCoherenceService.new(package, entry).execute
        end

        package.default!

        ::Packages::Npm::CreateMetadataCacheWorker.perform_async(package.project_id, package.name)

        ServiceResponse.success
      end

      private

      attr_reader :package_file

      def valid_package_file?
        package_file && !package_file.file.empty_size? && package&.npm? && package&.processing?
      end

      def with_package_json_entry
        package_file.file.use_open_file(unlink_early: false) do |open_file|
          Zlib::GzipReader.open(open_file.file_path) do |gz|
            found = false

            Gem::Package::TarReader.new(gz).seek(PACKAGE_JSON_ENTRY_PATH) do |entry|
              found = true
              yield entry
            end

            yield unless found
          end
        end
      end
    end
  end
end

==> gitlab_17.3.5.orig.tar.bz2: gitlab-foss-v17.3.5/app/services/packages/npm/process_package_file_service.rb <==
# frozen_string_literal: true

module Packages
  module Npm
    class ProcessPackageFileService
      ExtractionError = Class.new(StandardError)
      PACKAGE_JSON_ENTRY_REGEX = %r{^[^/]+/package.json$}
      MAX_FILE_SIZE = 4.megabytes

      delegate :package, to: :package_file

      def initialize(package_file)
        @package_file = package_file
      end

      def execute
        raise ExtractionError, 'invalid package file' unless valid_package_file?

        with_package_json_entry do |entry|
          raise ExtractionError, 'package.json not found' unless entry
          raise ExtractionError, 'package.json file too large' if entry.size > MAX_FILE_SIZE

          ::Packages::Npm::CheckManifestCoherenceService.new(package, entry).execute
        end

        package.default!

        ::Packages::Npm::CreateMetadataCacheWorker.perform_async(package.project_id, package.name)

        ServiceResponse.success
      end

      private

      attr_reader :package_file

      def valid_package_file?
        package_file && !package_file.file.empty_size? && package&.npm? && package&.processing?
      end

      def with_package_json_entry
        package_file.file.use_open_file(unlink_early: false) do |open_file|
          Zlib::GzipReader.open(open_file.file_path) do |gz|
            tar_reader = Gem::Package::TarReader.new(gz)

            entry_path = entry_full_name(tar_reader)
            yield unless entry_path.is_a?(String)

            tar_reader.rewind
            entry = tar_reader.find { |e| e.full_name == entry_path }

            yield entry
          end
        end
      end

      def entry_full_name(tar_reader)
        # We need to reverse the entries to find the last package.json file in the tarball,
        # as the last one is the one that's used by npm.
        # We cannot get the entry directly when using #reverse_each because
        # TarReader closes the stream after iterating over all entries
        tar_reader.reverse_each do |entry|
          break entry.full_name if entry.full_name.match?(PACKAGE_JSON_ENTRY_REGEX)
        end
      end
    end
  end
end
